@@ -27,15 +27,9 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef	HAVE_GETHRTIME
-  #include <sys/times.h>
-#endif
 #ifdef HAVE_GETTIMEOFDAY
   #include <sys/times.h>
 #endif
-#ifdef HAVE_GETRUSAGE
-  #include <sys/resource.h>
-#endif
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -51,7 +45,8 @@
 #endif
 
 static int open_device( const char *name, int minor );
-static int gather_random( byte *buffer, size_t *r_length, int level );
+static int gather_random( void (*add)(const void*, size_t, int), int requester,
+					  size_t length, int level );
 
 #ifdef IS_MODULE
 static void tty_printf(const char *fmt, ... )
@@ -61,38 +56,6 @@ static void tty_printf(const char *fmt, ... )
 #endif
 
 
-static void
-fast_poll( void (*add)(const void*, size_t, int) )
-{
-  #if HAVE_GETHRTIME
-    {	hrtime_t tv;
-	tv = gethrtime();
-	(*add)( &tv, sizeof(tv), 1 );
-    }
-  #elif HAVE_GETTIMEOFDAY
-    {	struct timeval tv;
-	if( gettimeofday( &tv, NULL ) )
-	    BUG();
-	(*add)( &tv.tv_sec, sizeof(tv.tv_sec), 1 );
-	(*add)( &tv.tv_usec, sizeof(tv.tv_usec), 1 );
-    }
-  #else /* use times */
-    {	struct tms buf;
-	times( &buf );
-	(*add)( &buf, sizeof buf, 1 );
-    }
-  #endif
-  #ifdef HAVE_GETRUSAGE
-    {	struct rusage buf;
-	if( getrusage( RUSAGE_SELF, &buf ) )
-	    BUG();
-	(*add)( &buf, sizeof buf, 1 );
-	memset( &buf, 0, sizeof buf );
-    }
-  #endif
-}
-
-
 
 /****************
  * Used to open the Linux and xBSD /dev/random devices
@@ -119,15 +82,15 @@ open_device( const char *name, int minor )
 
 
 static int
-gather_random( byte *buffer, size_t *r_length, int level )
+gather_random( void (*add)(const void*, size_t, int), int requester,
+					  size_t length, int level )
 {
     static int fd_urandom = -1;
     static int fd_random = -1;
     int fd;
     int n;
     int warn=0;
-    size_t length = *r_length;
-    /* note: we will always return the requested length */
+    byte buffer[768];
 
     if( level >= 2 ) {
 	if( fd_random == -1 )
@@ -139,7 +102,8 @@ gather_random( byte *buffer, size_t *r_length, int level )
 	    fd_urandom = open_device( NAME_OF_DEV_URANDOM, 9 );
 	fd = fd_urandom;
     }
-    do {
+
+    while( length ) {
 	fd_set rfds;
 	struct timeval tv;
 	int rc;
@@ -163,20 +127,21 @@ gather_random( byte *buffer, size_t *r_length, int level )
 	}
 
 	do {
-	    n = read(fd, buffer, length );
-	    if( n >= 0 && n > length ) {
+	    int nbytes = length < sizeof(buffer)? length : sizeof(buffer);
+	    n = read(fd, buffer, nbytes );
+	    if( n >= 0 && n > nbytes ) {
 		g10_log_error("bogus read from random device (n=%d)\n", n );
-		n = length;
+		n = nbytes;
 	    }
 	} while( n == -1 && errno == EINTR );
 	if( n == -1 )
 	    g10_log_fatal("read error on random device: %s\n", strerror(errno));
-	assert( n <= length );
-	buffer += n;
+	(*add)( buffer, n, requester );
 	length -= n;
-    } while( length );
+    }
+    memset(buffer, 0, sizeof(buffer) );
 
-    return 100; /* 100% useful at the requested level */
+    return 0; /* success */
 }
 
 
@@ -184,7 +149,7 @@ gather_random( byte *buffer, size_t *r_length, int level )
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.2 $)";
+const char * const gnupgext_version = "RNDLINUX ($Revision: 1.5 $)";
 
 static struct {
     int class;
@@ -192,7 +157,6 @@ static struct {
     void *func;
 } func_table[] = {
     { 40, 1, gather_random },
-    { 41, 1, fast_poll },
 };
 
 
