@@ -58,6 +58,9 @@
 #include "dynload.h"
 #endif
 
+#ifndef EAGAIN
+  #define EAGAIN  EWOULDBLOCK
+#endif
 
 #define GATHER_BUFSIZE		49152	/* Usually about 25K are filled */
 
@@ -426,7 +429,11 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 	    dataSources[i].pipeFD = fileno(dataSources[i].pipe);
 	    if (dataSources[i].pipeFD > maxFD)
 		maxFD = dataSources[i].pipeFD;
+	  #ifdef O_NONBLOCK /* Ohhh what a hack (used for Atari) */
 	    fcntl(dataSources[i].pipeFD, F_SETFL, O_NONBLOCK);
+	  #else
+	    #warning O_NONBLOCK is missing
+	  #endif
 	    FD_SET(dataSources[i].pipeFD, &fds);
 	    dataSources[i].length = 0;
 
@@ -702,6 +709,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
     /* now read from the gatherer */
     while( length ) {
 	int goodness;
+	ulong subtract;
 
 	if( read_a_msg( pipedes[0], &msg ) ) {
 	    g10_log_error("reading from gatherer pipe failed: %s\n",
@@ -735,7 +743,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	(*add)( msg.data, n, requester );
 
 	/* this is the trick how e cope with the goodness */
-	length -= (ulong)n * goodness / 100;
+	subtract = (ulong)n * goodness / 100;
+	/* subtract at least 1 byte to avoid infinite loops */
+	length -= subtract ? subtract : 1;
     }
 
     return 0;
@@ -746,7 +756,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.7 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.9 $)";
 
 
 static struct {
