@@ -24,6 +24,7 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
+#include <ctype.h>
 
 #include "errors.h"
 #include "iobuf.h"
@@ -255,31 +256,6 @@ parse_hash_header( const char *line )
 
 
 
-
-static unsigned
-trim_trailing_spaces( byte *line, unsigned len )
-{
-    byte *p, *mark;
-    unsigned n;
-
-    for(mark=NULL, p=line, n=0; n < len; n++, p++ ) {
-	if( strchr(" \t\r\n", *p ) ) {
-	    if( !mark )
-		mark = p;
-	}
-	else
-	    mark = NULL;
-    }
-
-    if( mark ) {
-	*mark = 0;
-	return mark - line;
-    }
-    return len;
-}
-
-
-
 /****************
  * Check whether this is a armor line.
  * returns: -1 if it is not a armor header or the index number of the
@@ -338,7 +314,7 @@ parse_header_line( armor_filter_context_t *afx, byte *line, unsigned len )
 
     if( *line == '\n' || ( len && (*line == '\r' && line[1]=='\n') ) )
 	return 0; /* empty line */
-    len = trim_trailing_spaces( line, len );
+    len = trim_trailing_ws( line, len );
     p = strchr( line, ':');
     if( !p || !p[1] ) {
 	log_error(_("invalid armor header: "));
@@ -499,7 +475,8 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	else {
 	    while( len < size && afx->buffer_pos < afx->buffer_len )
 		buf[len++] = afx->buffer[afx->buffer_pos++];
-	    if( afx->buffer_pos >= afx->buffer_len ) {
+	    if( afx->buffer_pos >= afx->buffer_len
+		&& !afx->not_dash_escaped ) {
 		buf[len++] = '\r';
 		buf[len++] = '\n';
 	    }
@@ -518,7 +495,8 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	}
 	if( !maxlen )
 	    afx->truncated++;
-	afx->buffer_len = trim_trailing_spaces( afx->buffer, afx->buffer_len );
+	if( !afx->not_dash_escaped )
+	    afx->buffer_len = trim_trailing_ws( afx->buffer, afx->buffer_len );
 	p = afx->buffer;
 	n = afx->buffer_len;
 
@@ -540,8 +518,6 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 		    putc('\n', stderr);
 		}
 		lastline = 1;
-		if( len >= 2 )
-		    len -= 2; /* remove the last CR,LF */
 		rc = -1;
 	    }
 	}
@@ -599,7 +575,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	      byte *buf, size_t size )
 {
     byte val;
-    int c, c2;
+    int c=0, c2; /*init c because gcc is not clever enough for the continue*/
     int checkcrc=0;
     int rc = 0;
     size_t n = 0;
@@ -610,11 +586,43 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
     idx = afx->idx;
     val = afx->radbuf[0];
     for( n=0; n < size; ) {
-	if( (c=iobuf_get(a)) == -1 )
-	    break;
+
+	if( afx->buffer_pos < afx->buffer_len )
+	    c = afx->buffer[afx->buffer_pos++];
+	else { /* read the next line */
+	    unsigned maxlen = MAX_LINELEN;
+	    afx->buffer_pos = 0;
+	    afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+					       &afx->buffer_size, &maxlen );
+	    if( !maxlen )
+		afx->truncated++;
+	    if( !afx->buffer_len )
+		break; /* eof */
+	    continue;
+	}
+
+      again:
 	if( c == '\n' || c == ' ' || c == '\r' || c == '\t' )
 	    continue;
 	else if( c == '=' ) { /* pad character: stop */
+	    /* some mailers leave quoted-printable encoded characters
+	     * so we try to workaround this */
+	    if( afx->buffer_pos+2 < afx->buffer_len ) {
+		int c1, c2, c3;
+		c1 = afx->buffer[afx->buffer_pos];
+		c2 = afx->buffer[afx->buffer_pos+1];
+		c3 = afx->buffer[afx->buffer_pos+2];
+		if( isxdigit(c1) && isxdigit(c2) && strchr( "=\n\r\t ", c3 )) {
+		    /* well it seems to be the case - adjust */
+		    c = isdigit(c1)? (c1 - '0'): (toupper(c1)-'A'+10);
+		    c <<= 4;
+		    c |= isdigit(c2)? (c2 - '0'): (toupper(c2)-'A'+10);
+		    afx->buffer_pos += 2;
+		    afx->qp_detected = 1;
+		    goto again;
+		}
+	    }
+
 	    if( idx == 1 )
 		buf[n++] = val;
 	    checkcrc++;
@@ -632,19 +640,32 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	}
 	idx = (idx+1) % 4;
     }
+
     for(i=0; i < n; i++ )
 	crc = (crc << 8) ^ crc_table[((crc >> 16)&0xff) ^ buf[i]];
     crc &= 0x00ffffff;
     afx->crc = crc;
     afx->idx = idx;
     afx->radbuf[0] = val;
+
     if( checkcrc ) {
 	afx->any_data = 1;
 	afx->inp_checked=0;
 	afx->faked = 0;
 	for(;;) { /* skip lf and pad characters */
-	    if( (c=iobuf_get(a)) == -1 )
-		break;
+	    if( afx->buffer_pos < afx->buffer_len )
+		c = afx->buffer[afx->buffer_pos++];
+	    else { /* read the next line */
+		unsigned maxlen = MAX_LINELEN;
+		afx->buffer_pos = 0;
+		afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+						   &afx->buffer_size, &maxlen );
+		if( !maxlen )
+		    afx->truncated++;
+		if( !afx->buffer_len )
+		    break; /* eof */
+		continue;
+	    }
 	    if( c == '\n' || c == ' ' || c == '\r'
 		|| c == '\t' || c == '=' )
 		continue;
@@ -664,8 +685,25 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		  case 2: val |= (c>>2)&15; mycrc |= val << 8;val=(c<<6)&0xc0;break;
 		  case 3: val |= c&0x3f; mycrc |= val; break;
 		}
-		if( (c=iobuf_get(a)) == -1 )
+		for(;;) {
+		    if( afx->buffer_pos < afx->buffer_len )
+			c = afx->buffer[afx->buffer_pos++];
+		    else { /* read the next line */
+			unsigned maxlen = MAX_LINELEN;
+			afx->buffer_pos = 0;
+			afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+							   &afx->buffer_size,
+								&maxlen );
+			if( !maxlen )
+			    afx->truncated++;
+			if( !afx->buffer_len )
+			    break; /* eof */
+			continue;
+		    }
 		    break;
+		}
+		if( !afx->buffer_len )
+		    break; /* eof */
 	    } while( ++idx < 4 );
 	    if( c == -1 ) {
 		log_error(_("premature eof (in CRC)\n"));
@@ -962,6 +1000,10 @@ armor_filter( void *opaque, int control,
 	if( afx->truncated )
 	    log_info(_("invalid armor: line longer than %d characters\n"),
 		      MAX_LINELEN );
+	/* issue an error to enforce dissemination of correct software */
+	if( afx->qp_detected )
+	    log_error(_("quoted printable character in armor - "
+			"probably a buggy MTA has been used\n") );
 	m_free( afx->buffer );
 	afx->buffer = NULL;
     }
