@@ -42,6 +42,7 @@ typedef struct {
     int hashes; 	    /* detected hash algorithms */
     int faked;		    /* we are faking a literal data packet */
     int truncated;	    /* number of truncated lines */
+    int qp_detected;
 
     byte *buffer;	    /* malloced buffer */
     unsigned buffer_size;   /* and size of this buffer */
@@ -66,6 +67,7 @@ typedef struct {
     byte *outbuf;
     unsigned outbufsize;
     int algo;	 /* compress algo */
+    int algo1hack;
 } compress_filter_context_t;
 
 
@@ -79,10 +81,17 @@ typedef struct {
 
 
 typedef struct {
-    int eof;
-    size_t idx;
-    size_t len;
-    byte buf[256];
+    byte *buffer;	    /* malloced buffer */
+    unsigned buffer_size;   /* and size of this buffer */
+    unsigned buffer_len;    /* used length of the buffer */
+    unsigned buffer_pos;    /* read position */
+    int truncated;	    /* number of truncated lines */
+    int clearsign;
+    int not_dash_escaped;
+    int escape_from;
+    MD_HANDLE md;
+    int pending_lf;
+    int pending_esc;
 } text_filter_context_t;
 
 
