@@ -1,5 +1,5 @@
 /* g10.c - The GnuPG utility (main for gpg)
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -43,6 +43,7 @@
 #include "i18n.h"
 #include "status.h"
 #include "g10defs.h"
+#include "hkp.h"
 
 #ifndef IS_G10MAINT
   #define IS_G10 1
@@ -83,6 +84,7 @@ enum cmd_and_opt_values { aNull = 0,
     aListKeys,
     aListSigs,
     aListSecretKeys,
+    aSendKeys,
     aExport,
     aExportAll,
     aExportSecret,
@@ -152,6 +154,7 @@ enum cmd_and_opt_values { aNull = 0,
     oNotDashEscaped,
     oEscapeFrom,
     oLockOnce,
+    oKeyServer,
 aTest };
 
 
@@ -183,6 +186,7 @@ static ARGPARSE_OPTS opts[] = {
     { aGenRevoke, "gen-revoke",256, N_("generate a revocation certificate")},
   #endif
     { aExport, "export"           , 256, N_("export keys") },
+    { aSendKeys, "send-keys"     , 256, N_("export keys to a key server") },
     { aExportAll, "export-all"    , 256, "@" },
     { aExportSecret, "export-secret-keys" , 256, "@" },
     { aImport, "import",      256     , N_("import/merge keys")},
@@ -229,6 +233,7 @@ static ARGPARSE_OPTS opts[] = {
     { oKeyring, "keyring"   ,2, N_("add this keyring to the list of keyrings")},
     { oSecretKeyring, "secret-keyring" ,2, N_("add this secret keyring to the list")},
     { oDefaultKey, "default-key" ,2, N_("|NAME|use NAME as default secret key")},
+    { oKeyServer, "keyserver",2, N_("|HOST|use this keyserver to lookup keys")},
     { oCharset, "charset"   , 2, N_("|NAME|set terminal charset to NAME") },
     { oOptions, "options"   , 2, N_("read options from file")},
 
@@ -637,6 +642,7 @@ main( int argc, char **argv )
 	  case aListPackets: set_cmd( &cmd, aListPackets); break;
 	  case aImport: set_cmd( &cmd, aImport); break;
 	  case aFastImport: set_cmd( &cmd, aFastImport); break;
+	  case aSendKeys: set_cmd( &cmd, aSendKeys); break;
 	  case aExport: set_cmd( &cmd, aExport); break;
 	  case aExportAll: set_cmd( &cmd, aExportAll); break;
 	  case aListKeys: set_cmd( &cmd, aListKeys); break;
@@ -785,6 +791,7 @@ main( int argc, char **argv )
 	  case oNotDashEscaped: opt.not_dash_escaped = 1; break;
 	  case oEscapeFrom: opt.escape_from = 1; break;
 	  case oLockOnce: opt.lock_once = 1; break;
+	  case oKeyServer: opt.keyserver_name = pargs.r.ret_str; break;
 
 	  default : pargs.err = configfp? 1:2; break;
 	}
@@ -1095,10 +1102,14 @@ main( int argc, char **argv )
 
       case aExport:
       case aExportAll:
+      case aSendKeys:
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
 	    add_to_strlist( &sl, *argv );
-	export_pubkeys( sl, (cmd == aExport) );
+	if( cmd == aSendKeys )
+	    hkp_export( sl );
+	else
+	    export_pubkeys( sl, (cmd == aExport) );
 	free_strlist(sl);
 	break;
 
