@@ -109,8 +109,11 @@ KBNODE make_mpi_comment_node( const char *s, MPI a );
 
 /*-- import.c --*/
 int import_keys( const char *filename, int fast );
+int import_keys_stream( IOBUF inp, int fast );
+
 /*-- export.c --*/
 int export_pubkeys( STRLIST users, int onlyrfc );
+int export_pubkeys_stream( IOBUF out, STRLIST users, int onlyrfc );
 int export_seckeys( STRLIST users );
 
 /* dearmor.c --*/
