@@ -38,6 +38,7 @@
 #include "status.h"
 #include "i18n.h"
 #include "trustdb.h"
+#include "hkp.h"
 
 /****************
  * Structure to hold the context
@@ -840,6 +841,10 @@ check_sig_and_print( CTX c, KBNODE node )
 	    (int)strlen(tstr), tstr, astr? astr: "?", (ulong)sig->keyid[1] );
 
     rc = do_check_sig(c, node, NULL );
+    if( rc == G10ERR_NO_PUBKEY && opt.keyserver_name ) {
+	if( !hkp_ask_import( sig->keyid ) )
+	    rc = do_check_sig(c, node, NULL );
+    }
     if( !rc || rc == G10ERR_BAD_SIGN ) {
 	char *us = get_long_user_id_string( sig->keyid );
 	write_status_text( rc? STATUS_BADSIG : STATUS_GOODSIG, us );
