@@ -35,6 +35,18 @@ else
     DIE="yes"
 fi
 
+
+if (gettext --version </dev/null 2>/dev/null | awk 'NR==1 { split($4,A,"\."); \
+    X=10000*A[1]+100*A[2]+A[3]; echo X; if( X >= 1035 ) exit 1; exit 0}')
+    then
+    echo "**Error**: You must have "\`gettext\'" installed to compile $PGM."
+    echo '           (version 0.10.35 or newer is required; get'
+    echo '            ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz)'
+    DIE="yes"
+fi
+
+
+
 if test "$DIE" = "yes"; then
     exit 1
 fi
