@@ -1,5 +1,5 @@
 /* [argparse.c wk 17.06.97] Argument Parser for option handling
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998,1999 Free Software Foundation, Inc.
  *  This file is part of GnuPG.
  *
  *  GnuPG is free software; you can redistribute it and/or modify
@@ -873,7 +873,7 @@ default_strusage( int level )
     switch( level ) {
       case 11: p = "foo"; break;
       case 13: p = "0.0"; break;
-      case 14: p = "Copyright (C) 1998 Free Software Foundation, Inc."; break;
+      case 14: p = "Copyright (C) 1999 Free Software Foundation, Inc."; break;
       case 15: p =
 "This program comes with ABSOLUTELY NO WARRANTY.\n"
 "This is free software, and you are welcome to redistribute it\n"
@@ -928,7 +928,7 @@ main(int argc, char **argv)
 
     while( ArgParse( &pargs, opts) ) {
 	switch( pargs.r_opt ) {
-	  case -1 : printf( "arg='%s'\n", pargs.r.ret_str); break;
+	  case -1 : printf( "arg=`%s'\n", pargs.r.ret_str); break;
 	  case 'v': opt.verbose++; break;
 	  case 'e': opt.echo++; break;
 	  case 'd': opt.debug++; break;
@@ -947,9 +947,9 @@ main(int argc, char **argv)
     if( opt.debug )
 	printf("  debug=%d\n", opt.debug );
     if( opt.outfile )
-	printf("  outfile='%s'\n", opt.outfile );
+	printf("  outfile=`%s'\n", opt.outfile );
     if( opt.crf )
-	printf("  crffile='%s'\n", opt.crf );
+	printf("  crffile=`%s'\n", opt.crf );
     if( opt.myopt )
 	printf("  myopt=%d\n", opt.myopt );
     if( opt.a_long_one )
